library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
library(Kendall)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "aemet-5km_mon/OUTPUTS/"

#Useful information
calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
regions <- loadGridData(dataset="aemet-5km_mon/aemet/aemet_clus_dm_5.nc",
                        var = "__xarray_dataarray_variable__")

#Load CPs information
load("WardCluster_Dendrogram_D2_50km.Rdata")
k=5
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1951:2020))]
cp <- loadGridData(dataset = "eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),years=1951:2020,var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}

#________________SUBSEASONAL (MONTHLY)________________#
season <- c("Jan"=1,"Feb"=2,"Mar"=3,"Apr"=4,"May"=5,"Jun"=6,
            "Jul"=7,"Aug"=8,"Sep"=9,"Oct"=10,"Nov"=11,"Dec"=12)

for(r in 1:5){
reg <- regions
reg$Data[which(regions$Data!=r)] <- 0
reg$Data[which(regions$Data==r)] <- 1

#____________________tasmin_______________________________#
#Load (monthly) surface data
nc <- loadGridData(dataset="aemet-5km_mon/tasmin_mon_aemet-5km_1951-2020.nc",var="tasmin")
for(y in 1:dim(nc$Data)[1]){
  nc$Data[y,,] <- nc$Data[y,,]*reg$Data
}

data <- list(); value <- NA ; sign <- NA
for(i in 1:12){
  mon_intann <- subsetGrid(nc,season=i)
  index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
    
  #Generating time-series for each CP
  for(l in 1:length(unique(vector_cluster))){
  v <- unique(vector_cluster)[l]
  cp2 <- cp
  cp2$Data[which(cp2$Data!=v)] <- 0
  cp2$Data[which(cp2$Data==v)] <- 1
  index.cp <- redim(aggregateGrid(redim(cp2),aggr.m = list(FUN="sum",na.rm=T)),drop=T)
  
  #Selecting the month/season of interest
  index.cp <- subsetGrid(index.cp,season=i)
  
  #Removing trend in CP frequency (if any)
  trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
  index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
  
  #Removing trend in surface variable (if any)
  trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
  index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
  
  value[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
  sign[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
  }
  
  data[[i]] <- data.frame(Index=value,
                          Season=rep(names(season)[i],length(value)),
                          CP=unique(vector_cluster),
                          Sign=sign)
}

datitos.cor <- do.call(rbind,data)
datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))

g <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
  scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                        aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
  ggtitle("TN")+
  geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
  geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)

#____________________tasmax_______________________________#
nc <- loadGridData(dataset="aemet-5km_mon/tasmax_mon_aemet-5km_1951-2020.nc",var="tasmax")
for(y in 1:dim(nc$Data)[1]){
  nc$Data[y,,] <- nc$Data[y,,]*reg$Data
}

data <- list(); value <- NA ; sign <- NA
for(i in 1:12){
  mon_intann <- subsetGrid(nc,season=i)
  index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
  
  #Generating time-series for each CP
  for(l in 1:length(unique(vector_cluster))){
    v <- unique(vector_cluster)[l]
    cp2 <- cp
    cp2$Data[which(cp2$Data!=v)] <- 0
    cp2$Data[which(cp2$Data==v)] <- 1
    index.cp <- redim(aggregateGrid(redim(cp2),aggr.m = list(FUN="sum",na.rm=T)),drop=T)
    
    #Selecting the month/season of interest
    index.cp <- subsetGrid(index.cp,season=i)
    
    #Removing trend in CP frequency (if any)
    trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
    index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
    
    #Removing trend in surface variable (if any)
    trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
    index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
    
    value[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
    sign[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
  }
  
  data[[i]] <- data.frame(Index=value,
                          Season=rep(names(season)[i],length(value)),
                          CP=unique(vector_cluster),
                          Sign=sign)
}

datitos.cor <- do.call(rbind,data)
datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))

h <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
  scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                        aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
  ggtitle("TX")+
  geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
  geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)

#____________________precip_______________________________#
nc <- loadGridData(dataset="aemet-5km_mon/prlr_mon_aemet-5km_1951-2020.nc",var="prlr")
for(y in 1:dim(nc$Data)[1]){
  nc$Data[y,,] <- nc$Data[y,,]*reg$Data
}

data <- list(); value <- NA ; sign <- NA
for(i in 1:12){
  mon_intann <- subsetGrid(nc,season=i)
  index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
  
  #Generating time-series for each CP
  for(l in 1:length(unique(vector_cluster))){
    v <- unique(vector_cluster)[l]
    cp2 <- cp
    cp2$Data[which(cp2$Data!=v)] <- 0
    cp2$Data[which(cp2$Data==v)] <- 1
    index.cp <- redim(aggregateGrid(redim(cp2),aggr.m = list(FUN="sum",na.rm=T)),drop=T)
    
    #Selecting the month/season of interest
    index.cp <- subsetGrid(index.cp,season=i)
    
    #Removing trend in CP frequency (if any)
    trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
    index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
    
    #Removing trend in surface variable (if any)
    trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
    index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
    
    value[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
    sign[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
  }
  
  data[[i]] <- data.frame(Index=value,
                          Season=rep(names(season)[i],length(value)),
                          CP=unique(vector_cluster),
                          Sign=sign)
}

datitos.cor <- do.call(rbind,data)
datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))

p <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
  scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                        aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
  ggtitle("PR")+
  geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
  geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)

#___
png(filename = paste(OUTPUTS,"Cor_aemet-5km_",v,"CPs_D4_Region",r,"_Subseasonal_TN.png"),res=300,width=4000,height=4000)
print(grid.arrange(p,g,h,nrow=1))
dev.off()
##################################################################################
season <- list("Winter"=c(12,1,2),"Spring"=c(3,4,5),"Summer"=c(6,7,8),"Autumn"=c(9,10,11),"Annual"=c(1:12))
